package com.cn.listener;

import com.cn.model.ChatGroup;
import org.jim.common.ImAio;
import org.jim.common.ImConst;
import org.jim.common.ImPacket;
import org.jim.common.ImSessionContext;
import org.jim.common.cache.redis.JedisTemplate;
import org.jim.common.packets.*;
import org.jim.server.enums.GroupStateEnum;
import org.jim.server.enums.ServiceAccountOfflineTypeEnum;
import org.jim.server.enums.VisitTypeEnum;
import org.jim.server.listener.ImGroupListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.tio.core.ChannelContext;

import java.util.Map;

/**
 * @author Darren
 * @date 2020/2/6 15:53
 */
public class IMChatGroupListener extends ImGroupListener {

    private static Logger logger = LoggerFactory.getLogger(IMChatGroupListener.class);

    private static JedisTemplate jedisTemplate = null;

    static {
        try {
            jedisTemplate = JedisTemplate.me();
        } catch (Exception e) {
            logger.info("JedisTemplate初始化失败!");
            e.printStackTrace();
        }
    }

    @Override
    public void onAfterUnbind(ChannelContext channelContext, String group) throws Exception {

        ImSessionContext imSessionContext = (ImSessionContext)channelContext.getAttribute();
        ExitGroupNotifyRespBody exitGroupNotifyRespBody = new ExitGroupNotifyRespBody();
        exitGroupNotifyRespBody.setGroup(group);
        Client client = imSessionContext.getClient();
        if(client == null){
            return;
        }
        User clientUser = client.getUser();
        if(clientUser == null) {
            return;
        }
        //访问类型 (SERVICEACCOUNT 客服 / CUSTOMER 客户 / VISITOR 游客)
        String visitType = clientUser.getVisitType();
        if( !VisitTypeEnum.SERVICEACCOUNT.equals(visitType) ){
            ChatGroup chatGroup = ChatGroup.dao.findById(group);
            if( null != chatGroup ){
                Integer serviceAccountType = chatGroup.getInt("service_account_type");
                // service_account_type  客服是否是离线状态 1: 是 0: 否
                if( ServiceAccountOfflineTypeEnum.NO.getKey() == serviceAccountType ){
                    //只能更改 客服 是在线时 的群组状态;  group_state 群组状态 1:在线 0:离线
                    chatGroup.set("group_state", GroupStateEnum.OFF_LINE.getKey()).update();
                }
            }

        }else{
            //移除客服容器中的 客服
            String serviceAccountContainerKey
                    =  clientUser.getExtras().getString("companyId") + ImConst.SEPARATOR
                    + clientUser.getExtras().getString("departmentId") + ImConst.SEPARATOR
                    + VisitTypeEnum.SERVICEACCOUNT.getKey() ;
            String userId = clientUser.getId();
            String[] userIdArray = {userId};
            jedisTemplate.hashDel(serviceAccountContainerKey,userIdArray);
        }

        User notifyUser = new User(clientUser.getId(),clientUser.getNick());
        exitGroupNotifyRespBody.setUser(notifyUser);

        RespBody respBody = new RespBody(Command.COMMAND_EXIT_GROUP_NOTIFY_RESP,exitGroupNotifyRespBody);
        ImPacket imPacket = new ImPacket(Command.COMMAND_EXIT_GROUP_NOTIFY_RESP, respBody.toByte());
        ImAio.sendToGroup(group, imPacket);
    }


}
