package com.cn.processor;

import com.cn.model.ChatGroup;
import com.cn.model.Conversation;
import com.cn.model.ConversationRecord;
import org.jim.common.ImAio;
import org.jim.common.packets.ChatBody;
import org.jim.common.packets.ChatType;
import org.jim.common.packets.User;
import org.jim.server.command.handler.processor.chat.DefaultAsyncChatMessageProcessor;
import org.jim.server.enums.VisitTypeEnum;
import org.jim.server.util.SnowflakeIdUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.tio.core.ChannelContext;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @author Darren
 * @date 2020/2/6 19:42
 */
public class IMChatAsyncChatMessageProcessor extends DefaultAsyncChatMessageProcessor {

    private static Logger log = LoggerFactory.getLogger(IMChatAsyncChatMessageProcessor.class);

    private static SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    @Override
    public void doHandler(ChatBody chatBody, ChannelContext channelContext) {

        log.info("聊天记录持久化... 开始:" + sdf.format(new Date()));

        //聊天类型(如 0:未知 1:公聊、2:私聊)
        Integer chatType = chatBody.getChatType();
        //消息类型;(如：0:text、1:image、2:voice、3:vedio、4:music、5:news)
        Integer msgType = chatBody.getMsgType();
        if( ChatType.CHAT_TYPE_PUBLIC.getNumber() == chatType ){
            //公聊
            // group_id 与 conversation_id一致
            Conversation conversation = Conversation.dao.findById(chatBody.getGroup_id());

            Map<String, Object> attrs = new HashMap<>();
            attrs.put("id", SnowflakeIdUtils.getInstance().nextId());
            attrs.put("chat_body_id",chatBody.getId());
            attrs.put("content",chatBody.getContent());
            attrs.put("is_recall",0);

            String fromId = chatBody.getFrom();
            // customerDepartmentId  visitorDepartmentId  serviceAccountRoleDepartmentId
            User user = ImAio.getUser(fromId);
            String visitType = user.getVisitType();
            if(VisitTypeEnum.CUSTOMER.getKey().equals(visitType)){
                // CUSTOMER 客户  发送的消息
                String companyId = user.getExtras().getString("customerId");
                attrs.put("from_id",Long.valueOf(companyId));
                attrs.put("to_id",conversation.getLong("service_account_id"));
                attrs.put("type",0);
            }else if(VisitTypeEnum.VISITOR.getKey().equals(visitType)){
                // VISITOR 游客  发送的消息
                String visitorId = user.getExtras().getString("visitorId");
                attrs.put("from_id",Long.valueOf(visitorId));
                attrs.put("to_id",conversation.getLong("service_account_id"));
                attrs.put("type",0);
            }else{
                //SERVICEACCOUNT 客服  发送的消息
                String serviceAccountId = user.getExtras().getString("serviceAccountId");
                attrs.put("from_id",Long.valueOf(serviceAccountId));
                attrs.put("to_id",conversation.getLong("user_id"));
                attrs.put("type",1);
            }
            attrs.put("conversation_id",user.getExtras().getLong("conversationId"));
            ConversationRecord.dao._setAttrs(attrs).save();
        }

        log.info("聊天记录持久化... 结束:" + sdf.format(new Date()));

    }
}
